pro gnomdrizz, map, rot=rot, reso_arcmin=reso_arcmin, pxsize=pxsize, fits=fits, display=display, mapfits=mapfits, hfits=hfits, drizzle_st=drizzle_st, NESTED=NESTED, online=online, ps=ps, title=title, ordering=ordering, undef=undef, pessimistic=pessimistic , hhealp=hhealp

;+
;NAME: GNOMDRIZZ
;
; PURPPOSE:
;   This routine performs the projection of an healpix map, provided as a vector, into a square map
;   given a WCS projection type.
;   It uses a drizzling method to compute the exact intersection of the input and output pixelizations.
;   This projection method ensures the conservation of the photometry.
;   By default, the TAN projection is used. If you want to use another projection type, provide your own
;   fits header as an input of the routine.
;   The call of the routine is similar to the call of the routine gnomview of the healpix library.
;
; Inputs:
;    - map = healpix map
;
; OPTIONAL INPUTS:
;    - rot = coordinates of the center of projected map
;    - reso_arcmin [FLOAT] = pixel resolution of the projected map in arcmin 
;    - pxsize [INTEGER] = pixel size of the projected map (square map is the unique option) 
;    - hfits [STRING ARRAY] = fits header of the projected map, if you want to specify your own projection type
;    - drizzle_st = structure providing the information of the projection from healpix to fits maps.
;      this is provided by the routine itself, and can be used for other runs on the same
;      kind of projection to spare computating time.
;
; OPTIONAL OUTPUTS:
;    - fits [STRING] = name of the output fits file containing the projected map
;    - mapfits [FLOAT] = name of the output matrix containing the projected map
;    - hfits [STRING ARRAY] = header fits of the projected map 
;    - drizzle_st = structure providing the information of the projection from healpix to fits maps.
;
; OPTIONAL KEYWORD:
;    - display = it performs online display
;    - NESTED
;    - online
;    - ps = to produce a ps file
;    - title = used to display
;    - ordering
;    - undef = to set the undefined value, if different from the healpix standard.
;    - pessimistic
;    - hhealp
;
; AUTHORS:
;    J.-P. Bernard (IRAP)
;    L. Montier (IRAP)
;    A. Sauve (IRAP)
;
;-


if not keyword_set(rot) then rot=[0.,0.]
if not keyword_set(pxsize) then pxsize = 500
if not keyword_set(reso_arcmin) then reso_arcmin = 1


if keyword_set(undef) then begin
    defsysv, '!drz_undef', undef, 1
endif else begin
    defsysv, '!drz_undef', !healpix.bad_value, 1
endelse

;init header fits
;*****************
if not keyword_set(hfits) then begin
    mkhdr, hfits, fltarr(2,2)
    sxaddpar, hfits, 'NAXIS1', pxsize
    sxaddpar, hfits, 'NAXIS2', pxsize
    sxaddpar, hfits, 'CTYPE1', 'GLON-TAN'
    sxaddpar, hfits, 'CRVAL1', rot[0]
    sxaddpar, hfits, 'CRPIX1', pxsize/2.+0.5
    sxaddpar, hfits, 'CDELT1', reso_arcmin/(-60.)
    sxaddpar, hfits, 'CTYPE2', 'GLAT-TAN'
    sxaddpar, hfits, 'CRVAL2', rot[1]
    sxaddpar, hfits, 'CRPIX2', pxsize/2.+0.5
    sxaddpar, hfits, 'CDELT2', reso_arcmin/(60.)
endif else begin
    pxsize = sxpar(hfits,'NAXIS1')
endelse


;init header healpix
;******************** 
if not keyword_set(ordering) then begin
    if keyword_set(NESTED) then begin
        ordering = 'NESTED'
    endif else begin
        ordering = 'RING'
    endelse
endif 

if not keyword_set(hhealp) then begin
    coordsys = 'G     '
    nside = npix2nside(n_elements(map))
    hhealp = drz_make_healp_header(nside,coordsys=coordsys,ordering=ordering)
endif else begin
    nside = npix2nside(n_elements(map))
endelse


;compute drizzle structure
;*************************
if not keyword_set(drizzle_st) then begin
    drizzle_st = drz_healp_drizzle5(hhealp,hfits,/jd,/silent)
    drizzle_st = drz_healp_drizzle_geom_invert(drizzle_st)
endif 


;compute fits map
;*****************
mapfits = drz_healp2fits_drizzle(map,drizzle_st,pessimistic=pessimistic)


;display
;********
if keyword_set(display) then begin
    loadct, 39
    undef = !healpix.bad_value
    pdef = where(mapfits ne undef,ndef)
    window, xsize= pxsize, ysize= pxsize + 200
    tv, bytscl(mapfits,min=min(mapfits(pdef)),max=max(mapfits(pdef))), 0, 100
endif


;store fits
;***********
if keyword_set(fits) then begin
    undef = !healpix.bad_value
    pdef = where(mapfits ne undef,ndef)
    if ndef ne 0 then begin
        sxaddpar, hfits, 'DATAMIN', min(mapfits(pdef))
        sxaddpar, hfits, 'DATAMAX', max(mapfits(pdef))
    endif
    writefits, fits, mapfits, hfits
endif

heap_gc  ; helps idl free its unused memory

end
